#! /usr/bin/env python3
# -*- coding:utf-8 -*-

import os, sys, time, re, socket

Debug = False
printEmailAlerts = False

os.environ['DJANGO_SETTINGS_MODULE'] = 'xc.xc.settings'
sys.path.append('/var/lib/ai-and-it/xc')

fromEmail = 'usense@example.com'
recList = []

def getEmailInfo():
    try:
        import xc.xc.settings
        global fromEmail, recList
        fromEmail = xc.xc.settings.DEFAULT_FROM_EMAIL
        recList = xc.xc.settings.EMAIL_RECIPIENTS
    except ImportError as e:
        print(e)
        print(f'sys.path: {sys.path}')
        pass

def initModule():
    getEmailInfo()

valinfo = {}
def loadValueInfo(fname):
    fdata = open(fname).read()

    fdata = fdata[1:]

    parts = fdata.split(';')
    setValueInfo(parts)

def setValueInfo(parts):
    names = []
    units = []

    rc = re.compile('(.*) \((.*)\)')
    for k in parts:
        m = rc.match(k.strip())
        names.append(m.group(1))
        units.append(m.group(2))

    valinfo['vals2unit'] = { names[i]: units[i] for i in range(len(names)) }
    valinfo['unit2vals'] = { units[i]: names[i] for i in range(len(names)) }

lastCalls = {}
def send_email(subj, body):
    if len(recList) == 0:
        print('No recipients configured')
        return
    if Debug:
        print('Sending email to', recList, 'from', fromEmail)
        print('Subject: %s' % (subj,))
        sys.stdout.flush()
    try:
        from django.core.mail import send_mail
    except:
        print('Failed to import send_mail from django')
        return
    try:
        send_mail(
            subject=subj,
            message=body,
            from_email=fromEmail,
            recipient_list=recList,
            fail_silently=False,
        )
        if Debug:
            print('Email to', recList, 'sent succesfully')
    except BaseException as e:
        lastCalls['failed'] = True
        print('send email failed:', e)
        sys.stdout.flush()

def selCharIndicator(type):
    if type == -2:
        c = '\u2193'
    elif type == -1:
        c = '\u2193'
    elif type == 1:
        c = '\u2191'
    elif type == 2:
        c = '\u2191'
    return c

def send_email_alert(actions):

    if 'failed' in lastCalls:
        return

    if 'vals2unit' not in valinfo:
        loadValueInfo('/var/lib/xc/usense/um-header.csv')

    t = time.time()

    doSend = False
    for act in actions:
        if not act['source'] in lastCalls:
            lastCalls[act['source']] = {'t': 0, 'n': 0}
        if t - lastCalls[act['source']]['t'] > 300:
            doSend = True

    if doSend:
        for act in actions:
            lastCalls[act['source']] = {'t': t, 'n': lastCalls[act['source']]['n'] + 1}
        if Debug:
            print('sending alert')
            print(lastCalls)
            sys.stdout.flush()
        values = [ act['value'] for act in actions ]
        units = [ valinfo['vals2unit'][act['source']] for act in actions ]
        sources = [ act['source'] for act in actions ]
        types = [ act['type'] for act in actions ]
        types = [ selCharIndicator(f) for f in types ]
        alertInds = [ sources[k] + types[k] for k in range(len(types)) ]
        alertIndsL = [ ' - %s%s: %g %s' % (sources[k], types[k], values[k], units[k]) for k in range(len(types)) ]

        emailSubj = '%s Alerts: %s' % (socket.gethostname(), ', '.join(alertInds),)
        emailText = """On %s, the following alert events have occured:

%s
                   """ % (socket.gethostname(), '\n'.join(alertIndsL),)

        if len(recList) == 0:
            if printEmailAlerts:
                print(emailSubj)
                print(emailText)
            return

        send_email(emailSubj, emailText)

def send_test_email():
    global Debug
    Debug = True
    initModule()
    send_email(
        'uSense email sending test (%s)' % (socket.gethostname(),),
        'You received this message because a test email was sent from the fionec uSense device with hostname "%s".\n\nWhen the uSense device is connected to your local network, it can be reached by invoking http://%s.local/' % (socket.gethostname(), socket.gethostname())
    )


if __name__ == "__main__":
    send_test_email()
