#! /usr/bin/env python3

import time
import math

from iim42652_spi.iim42652 import getIIM42652
from iim42652_spi import table
from iim42652_spi import timer

regWhoami = 0
valWhoami = 0xEA

regReset = 0x06
valReset = 0x41


def run():

  (iim42652, args) = getIIM42652()

  print(iim42652)

  i = 0
  i0 = ilast = 0
  t0 = tlast = time.time()

  header = ('Time (s)',
            'Acc_x (g)', 'Acc_y (g)', 'Acc_z (g)',
            'Gyr_x (dps)', 'Gyr_y (dps)', 'Gyr_z (dps)', 'Temp (°C)')
  outf = open(args.outfile, 'w')
  outt = table.Table(outf, header, args.precision)
  outt.writeHeader()

  tout = timer.WTimer()
  tout.click()
  tlastp = tout.time

  while True:
    tout.click()

    iim42652.wait4data()
    data = iim42652.readConfigured()

    timeval = tout.elaps
    data = (timeval,) + data

    outt.writeValues(data)

    if tout.elapsed('info', 1):
      cnt = i - ilast
      print("%d: %s %g reads/s" % (i, data, cnt / (tout.time - tlastp)))
      tlastp = tout.time
      ilast = i

    i += 1

    if args.maxreads > 0 and i > args.maxreads:
      break
    if args.maxreadt >= 0 and tout.elaps > args.maxreadt:
      break

  print('accel')
  print(iim42652.spi.get3x2(0x2D))
  print(iim42652.readAccel())

  print('gyro')
  print(iim42652.spi.get3x2(0x33))
  print(iim42652.readGyro())

  print('temp')
  print(iim42652.get2(iim42652.TEMP_OUT_H))
  print(iim42652.readTemp())

if __name__ == "__main__":
  run()

# Local Variables:
# coding: utf-8
# End:
