#! /usr/bin/env python3

from . import spitool
import time
import datetime
import argparse
import math
import re

from iim42652_spi.iim42652 import IIM42652

def parseargs():
  parser = argparse.ArgumentParser(description='Erzeuge Umspul-Konfiguration.')

  parser.add_argument('--bus', dest='bus', type=int, default=0, help='SPI bus')
  parser.add_argument('--dev', dest='dev', type=int, default=0, help='SPI device')
  parser.add_argument('--speed', dest='speed', type=int, default=int(1e6), help='SPI speed')

  parser.add_argument('--trace', action='store_true', help='Trace SPI')
  parser.add_argument('--reset', action='store_true', help='Reform initial reset')

  parser.add_argument('--odr', '-r', type=int, default=500, help='Trace SPI')

  parser.add_argument('--tmax', type=float, default=2, help='Trace SPI')
  parser.add_argument('--tmin', type=float, default=3, help='Trace SPI')
  parser.add_argument('--tavg', type=float, default=3, help='Trace SPI')
  parser.add_argument('--jerk', type=float, default=17, help='Trace SPI')
  parser.add_argument('--tol', type=float, default=2, help='Trace SPI')

  args = parser.parse_args()
  return args

def run():
  import locale
  locale.setlocale(locale.LC_ALL, '')

  args = parseargs()
  if args.trace:
    print(args)
    print(f'locale: {locale.getlocale()}')
    IIM42652.DEBUG = True
    spitool.DEBUG = True

  spi = spitool.SPIConn(args.bus, args.dev, args.speed)

  iim42652 = IIM42652(spi)
  if not iim42652.connected:
    print("device not found")
    return

  if args.reset:
    iim42652.reset()

  odr = 50
  if args.odr:
    odr = args.odr

  #
  iim42652['accelODR'] = odr
  iim42652['powerAccel'] = 'lowpower'

  #
  iim42652['accelDec2M2FiltOrdInt'] = 2
  iim42652['accelUiFiltBWInt'] = 4

  time.sleep(1e-3)

  #
  iim42652['tapTMin'] = args.tmin
  iim42652['tapTAvg'] = args.tavg
  iim42652['tapTMax'] = args.tmax

  iim42652['tapMaxPeakTol'] = args.tol
  iim42652['tapMinJerkThresh'] = args.jerk

  time.sleep(50e-3)

  iim42652['tapEnable'] = True


  inter = 0.9 / odr
  ntaps = 0

  propInt3 = getattr(IIM42652.Register, 'INT_STATUS3')
  propApex4 = getattr(IIM42652.Register, 'APEX_DATA4')

  print(f'tap detecton active')
  while True:
    intint = iim42652['INT_STATUS3']
    ints = propInt3.splitr(iim42652.register, intint)
    if ints['tapDetInt']:
      t1 = datetime.datetime.now()
      tapInfoInt = iim42652['APEX_DATA4']
      tapInfo = propApex4.splitr(iim42652.register, tapInfoInt)

      tapt, tapn, tapa, tapd = (tapInfo[c] for c in ['tapNum', 'tapNumInt', 'tapAxis', 'tapDir'])
      doubleTapTiming = iim42652['doubleTapTiming']
      print(f'{t1.strftime("%x %X.%f")} tap: {tapt} ({tapn}) axis: {tapa} dir: {tapd} interv: {doubleTapTiming}')

    time.sleep(inter)

if __name__ == "__main__":
  run()
