#
# https://pymotw.com/3/socket/uds.html
#

import socket
import sys
import os
import traceback

from . import log

class MySocket:
    def __init__(self, addr, sock):
        self.server_address = addr
        self.sock = sock
    def getsockopt(self, opt):
        return self.sock.getsockopt(socket.SOL_SOCKET, opt)

    def getsockopts(self, opts):
        res = {}
        for i in opts:
            res[i] = self.getsockopt(getattr(socket, i))
        return res

    def __str__(self):
        s = "Socket %s" % (self.server_address,)

        to = self.sock.gettimeout()
        blo = self.sock.getblocking()
        s += "\nTimeout: %g (%d)" % (to if to is not None else 0,blo)

        s += "\nOpts: %s" % (self.getsockopts({'SO_ERROR',
                                             'SO_KEEPALIVE',
                                             'SO_LINGER',
                                             }),)

        return s

class LineSocket:

    server_address = '/run/usense/linesocket-%d.sock' % (os.getpid(),)

    connections = []

    sndBufferSize = None

    def __init__(self, sockName=None):
        if sockName:
            self.server_address = sockName

    def openSocket(self, nconnections):

        # Make sure the socket does not already exist
        try:
            os.unlink(self.server_address)
        except OSError:
            if os.path.exists(self.server_address):
                raise

        # Create a UDS socket
        sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

        # Bind the socket to the address
        log.msg('bind to socket %s' % (self.server_address,))
        sock.bind(self.server_address)

        os.chmod(self.server_address, 0o777)

        # Listen for incoming connections
        sock.listen(1)

        self.sock = sock

        while len(self.connections) < nconnections:
            # Wait for a connection
            #            log.msg('waiting for a connection...')
            connection, client_address = sock.accept()
            #            log.msg('connection')

            if self.sndBufferSize:
                bsz = connection.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
                log.msg('Connection SNDBUF is set to %d' % (bsz,))
                connection.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, self.sndBufferSize)
                bsz = connection.getsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF)
                log.msg('Connection SNDBUF is set to %d' % (bsz,))

            connection.setblocking(False)
            log.msg('new connection from %s: blocking %s, timeout %s' %(client_address, connection.getblocking(), connection.gettimeout()))
            self.connections += [{'conn': connection, 'addr': client_address}]

        sock.close()


    def transmitb(self, line):
        for k in range(len(self.connections)):
            try:
                self.connections[k]['conn'].sendall(line)
            except BlockingIOError as ex:
                #log.msg('transmitb would block')
                raise(ex)
            except BaseException as ex:
                log.error('Failed to send to socket', ex)
                traceback.print_exc()
                for k in range(len(self.connections)):
                    self.connections[k]['conn'].close()
                self.connections = {}
                self.sock.close()
                raise(ex)

    def transmit(self, line):
        self.transmitb(line.encode('utf8'))

    def write(self, line):
        self.transmit(line)

    def close(self):
        for k in range(len(self.connections)):
            self.connections[k]['conn'].close()

    def getblocking(self):
        return self.sock.getblocking()

    def __str__(self):
        return str(MySocket(self.server_address, self.sock))
