import time

class Timer:
  _tnow = _t0 = _telapse = 0

  def __init__(self, text="{:.3f} s", **kw):
    self.text = text
    self.reset()

  def __enter__(self):
    self.start()

  def __exit__(self, *args):
    self.stop()
    print(self)

  def reset(self):
    self._t0 = time.time()
    self._telapse = 0

  def start(self):
    now = time.time()
    self._tnow = now

  def stop(self):
    now = time.time()
    self._telapse += now - self._tnow
    self._tnow = now

  @property
  def elaps(self):
    return self._telapse

  @property
  def time(self):
    return self._tnow

  def __str__(self):
    return self.text.format(self._telapse)


class WTimer:
  _tnow = _t0 = _telapse = _tlast = 0
  stamps = {}

  def __init__(self):
    self.reset()

  def reset(self):
    self._t0 = time.time()
    self.click()

  def click(self):
    now = time.time()
    self._tlast = now - self._tnow
    self._tnow = now
    self._telapse = self._tnow - self._t0

  def elapsInter(self, name):
    if name not in self.stamps:
      return None
    return self._tnow - self.stamps[name]

  def elapsed(self, name, per):
    if name not in self.stamps:
      self.stamps[name] = self._tnow
    if self._tnow - self.stamps[name] >= per:
      self.stamps[name] = self.stamps[name] + per
      return True
    else:
      return False

  @property
  def start(self):
    return self._t0

  @property
  def elaps(self):
    return self._telapse

  @property
  def time(self):
    return self._tnow

  @property
  def last(self):
    return 1e3 * self._tlast

  @property
  def now(self):
    ts = time.strftime('%Y-%m-%d %H:%M:%S:{}'.format(int((self.time*1e3)%1000)), time.gmtime(self.time))
    return (ts, self.elaps, self.last)

  def __str__(self):
    return "%s, %.3f, %.3f" % self.now
